function GK = globalK(node,elem,ndim,mat,constrainttype)
% calculate the global stiffness matrix,mat
% output: GK

% calculate the geometric quantity for each element
aux = auxgeometry(node,elem);
centroid = aux.centroid;  
diameter = aux.diameter;  
area = aux.area;

sumElem = size(elem,1); % the number of element
sumNode = size(node,1);

elemLen = cellfun('length',elem); 
nnz = sum((2*elemLen).^2);
NsumNode = sumNode;
ii = zeros(nnz,1); jj = zeros(nnz,1); ss = zeros(nnz,1); 

% calculate element stiffness matrix

ia = 0;
for n = 1:sumElem
    AK = elemK(n,centroid,diameter,area,node,elem,ndim,mat,constrainttype);
    AB = reshape(AK',1,[]);
    index = elem{n};

    % --------- assembly index for ellptic projection -----------
    indexDof = [index, index+sumNode];  
    Ndof = length(indexDof);
    ii(ia+1:ia+Ndof^2) = reshape(repmat(indexDof, Ndof, 1), [], 1);
    jj(ia+1:ia+Ndof^2) = repmat(indexDof(:), Ndof, 1);
    ss(ia+1:ia+Ndof^2) = AB(:);
    ia = ia + Ndof^2;
    
end

GK = sparse(ii,jj,ss,NsumNode*2,NsumNode*2);